/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qlayout.h>
//Added by qt3to4:
#include <QVBoxLayout>
#include <Q3Frame>

#include <klocale.h>
#include <kiconloader.h>

#include <kdebug.h>

#include "ksprefs.moc"
#include "page_general.h"
#include "page_colors.h"
#include "page_startup.h"
#include "page_rmbmenu.h"
#include "page_servchan.h"
#include "page_font.h"
#include "page_autoconnect.h"
#include "page_irccolors.h"
#include "page_looknfeel.h"
#include "page_shortcuts.h"

KSPrefs::KSPrefs(QWidget * parent, const char * name):
    KPageDialog(parent)
{
    setButtons(KDialog::Help | KDialog::Ok | KDialog::Apply | KDialog::Cancel | KDialog::Default);
	setDefaultButton(KDialog::Ok);
	setFaceType(List);
	setCaption(i18n("Configure KSirc"));
    setAttribute( Qt::WA_DeleteOnClose );
    QFrame *itemLooknFeel= new QFrame();
	pageItemLooknFeel = new KPageWidgetItem( itemLooknFeel, i18n( "Look and Feel" ) );
	pageItemLooknFeel->setHeader(i18n( "Controls how kSirc looks" ));
	pageItemLooknFeel->setIcon(BarIcon( "ksirc", K3Icon::SizeMedium ) );
	addPage(pageItemLooknFeel);
    
    QFrame *itemGeneral= new QFrame();
    pageItemGeneral = new KPageWidgetItem(  itemGeneral, i18n( "General" ) );
    pageItemGeneral->setHeader(i18n( "General KSirc Settings" ));
    pageItemGeneral->setIcon(BarIcon( "ksirc", K3Icon::SizeMedium ) );
    addPage(pageItemGeneral);
 
	QFrame *itemStartup  = new QFrame();
	pageItemStartup = new KPageWidgetItem(itemStartup, i18n( "Startup" ) );
    pageItemStartup->setHeader(i18n( "KSirc Startup Settings" ));
    pageItemStartup->setIcon(BarIcon( "gear", K3Icon::SizeMedium ) );
    addPage(pageItemStartup);

	
	QFrame *itemColors   = new QFrame();
    pageItemColors = new KPageWidgetItem(itemColors, i18n( "Colors" ) );
    pageItemColors->setHeader(i18n( "KSirc Color Settings" ));
    pageItemColors->setIcon(BarIcon( "colors", K3Icon::SizeMedium ) );
    addPage(pageItemColors);

    QFrame *itemIRCColors   = new QFrame();
    pageItemIRCColors = new KPageWidgetItem(itemIRCColors, i18n( "IRC Colors" ) );
    pageItemIRCColors->setHeader(i18n( "KSirc IRC Color Settings" ));
    pageItemIRCColors->setIcon(BarIcon( "colors", K3Icon::SizeMedium ) );
    addPage(pageItemIRCColors);

    QFrame *itemRMBMenu  = new QFrame();
    pageItemRMBMenu = new KPageWidgetItem(itemRMBMenu, i18n( "User Menu" ) );
    pageItemRMBMenu->setHeader(i18n( "User Menu Configuration" ));
    pageItemRMBMenu->setIcon(BarIcon( "", K3Icon::SizeMedium ) );
    addPage(pageItemRMBMenu);

    QFrame *itemServChan = new QFrame();
    pageItemServChan = new KPageWidgetItem(itemServChan, i18n( "Server/Channel" ) );
    pageItemServChan->setHeader(i18n( "Server/Channel Configuration" ));
    pageItemServChan->setIcon(BarIcon( "", K3Icon::SizeMedium ) );
    addPage(pageItemServChan);

    QFrame *itemAutoConnect = new QFrame();
    pageItemAutoConnect = new KPageWidgetItem(itemAutoConnect, i18n( "Auto Connect" ) );
    pageItemAutoConnect->setHeader(i18n( "Auto Connect Configuration" ));
    pageItemAutoConnect->setIcon(BarIcon( "", K3Icon::SizeMedium ) );
    addPage(pageItemAutoConnect);

    QFrame *itemFont = new QFrame();
    pageItemFont = new KPageWidgetItem(itemFont, i18n( "Fonts" ) );
    pageItemFont->setHeader(i18n( "Font Settings" ));
    pageItemFont->setIcon(BarIcon( "fonts", K3Icon::SizeMedium ) );
    addPage(pageItemFont);

    QFrame *itemShortcuts = new QFrame();
    pageItemShortcuts = new KPageWidgetItem(itemShortcuts, i18n( "Shortcuts" ) );
    pageItemShortcuts->setHeader(i18n( "Shortcut Configuration" ));
    pageItemShortcuts->setIcon(BarIcon( "", K3Icon::SizeMedium ) );
    addPage(pageItemShortcuts);

    QVBoxLayout *fontTopLayout  = new QVBoxLayout( itemFont );
    fontTopLayout->setSpacing( 6 );
    fontTopLayout->setMargin( 0 );


    QVBoxLayout *looknfeelTopLayout= new QVBoxLayout( itemLooknFeel );
    looknfeelTopLayout->setSpacing( 6 );
    looknfeelTopLayout->setMargin( 0 );
    QVBoxLayout *generalTopLayout  = new QVBoxLayout( itemGeneral );
    generalTopLayout->setSpacing( 6 );
    generalTopLayout->setMargin( 0 );
    QVBoxLayout *startupTopLayout  = new QVBoxLayout( itemStartup );
    startupTopLayout->setSpacing( 6 );
    startupTopLayout->setMargin( 0 );
    QVBoxLayout *colorsTopLayout   = new QVBoxLayout( itemColors );
    colorsTopLayout->setSpacing( 6 );
    colorsTopLayout->setMargin( 0 );
    QVBoxLayout *ircColorsTopLayout   = new QVBoxLayout( itemIRCColors );
    ircColorsTopLayout->setSpacing( 6 );
    ircColorsTopLayout->setMargin( 0 );

    QVBoxLayout *rmbMenuTopLayout  = new QVBoxLayout( itemRMBMenu );
    rmbMenuTopLayout->setSpacing( 6 );
    rmbMenuTopLayout->setMargin( 0 );
    QVBoxLayout *servChanTopLayout = new QVBoxLayout( itemServChan );
    servChanTopLayout->setSpacing( 6 );
    servChanTopLayout->setMargin( 0 );
    QVBoxLayout *autoConnectTopLayout = new QVBoxLayout( itemAutoConnect );
    autoConnectTopLayout->setSpacing( 6 );
    autoConnectTopLayout->setMargin( 0 );
    QVBoxLayout *autoShortcutsTopLayout = new QVBoxLayout( itemShortcuts );
    autoShortcutsTopLayout->setSpacing( 6 );
    autoShortcutsTopLayout->setMargin( 0 );

    pageLooknFeel= new PageLooknFeel( itemLooknFeel );
    pageGeneral  = new PageGeneral( itemGeneral );
    pageStartup  = new PageStartup( itemStartup );
    pageColors   = new PageColors( itemColors );
    pageIRCColors   = new PageIRCColors( itemIRCColors );
    pageFont     = new PageFont( itemFont );
    pageRMBMenu  = new PageRMBMenu( itemRMBMenu );
    pageServChan = new PageServChan( itemServChan );
    pageAutoConnect = new PageAutoConnect( itemAutoConnect );
    pageShortcuts = new PageShortcuts( itemShortcuts );

    connect(pageLooknFeel, SIGNAL( modified() ), SLOT( modified() ) );
    connect(pageGeneral, SIGNAL( modified() ), SLOT( modified() ) );
    connect(pageStartup, SIGNAL( modified() ), SLOT( modified() ) );
    connect(pageColors, SIGNAL( modified() ), SLOT( modified() ) );
    connect(pageIRCColors, SIGNAL( modified() ), SLOT( modified() ) );
    connect(pageFont, SIGNAL( modified() ), SLOT( modified() ) );
    connect(pageRMBMenu, SIGNAL( modified() ), SLOT( modified() ) );
    connect(pageServChan, SIGNAL( modified() ), SLOT( modified() ) );
    connect(pageAutoConnect, SIGNAL( modified() ), SLOT( modified() ) );
    connect(pageShortcuts, SIGNAL( modified() ), SLOT( modified() ) );

    connect(this, SIGNAL( applyClicked() ), this, SLOT( saveConfig() ) );
    connect(this, SIGNAL( okClicked() ), this, SLOT( saveConfig() ) );
    connect(this, SIGNAL( defaultClicked() ), this, SLOT(defaultConfig() ) );

    looknfeelTopLayout->addWidget( pageLooknFeel );
    generalTopLayout->addWidget( pageGeneral );
    startupTopLayout->addWidget( pageStartup );
    colorsTopLayout->addWidget( pageColors );
    ircColorsTopLayout->addWidget( pageIRCColors );
    fontTopLayout->addWidget( pageFont );
    rmbMenuTopLayout->addWidget( pageRMBMenu );
    servChanTopLayout->addWidget( pageServChan );
    autoConnectTopLayout->addWidget( pageAutoConnect );
    autoShortcutsTopLayout->addWidget( pageShortcuts );
	//enableButtonSeperator( true );

    readConfig();
    enableButtonApply( false );

    pSCDirty = 0;
    pACDirty = 0;
}

KSPrefs::~KSPrefs()
{
}

void KSPrefs::readConfig()
{
	// apply by calling readConfig in each page

        pageLooknFeel->readConfig();
	pageGeneral->readConfig();
        pageColors->readConfig();
        pageIRCColors->readConfig();
	pageStartup->readConfig();
	pageFont->readConfig();
	pageRMBMenu->readConfig();
        pageServChan->readConfig();
	pageAutoConnect->readConfig();
        pageShortcuts->readConfig();

}

void KSPrefs::saveConfig()
{
	// apply by calling saveConfig in each page
	// use setDirty flag for each page and
	// emit update() appropriate

    if ( dirty & KSOptions::General )
        pageGeneral->saveConfig();
    if ( dirty & KSOptions::Startup )
        pageStartup->saveConfig();
    if ( dirty & KSOptions::Colors )
        pageColors->saveConfig();
    if ( dirty & KSOptions::Colors )
        pageIRCColors->saveConfig();
    if ( dirty & KSOptions::Colors )
        pageFont->saveConfig();
    if ( dirty & KSOptions::RMBMenu )
        pageRMBMenu->saveConfig();
    if ( dirty & pSCDirty )
        pageServChan->saveConfig();
    if ( dirty & pACDirty )
	pageAutoConnect->saveConfig();
    if ( dirty & pLFDirty )
        pageLooknFeel->saveConfig();
    if ( dirty & pShortDirty )
        pageShortcuts->saveConfig();


    ksopts->save( dirty );
    emit update( dirty );
    enableButtonApply( false );
}

void KSPrefs::showConnectPage()
{
    setCurrentPage(pageItemAutoConnect);
}

void KSPrefs::defaultConfig()
{
	// apply by calling defaultConfig in current page
	KPageWidgetItem *current = currentPage();
	if(current == pageItemLooknFeel)
    {
        kDebug(5008) << "Doing looknfeel" << endl;
        pageLooknFeel->defaultConfig();
	}
	else if(current == pageItemGeneral)
	{
        kDebug(5008) << "Doing general" << endl;
        pageGeneral->defaultConfig();
	}
	else if(current == pageItemStartup)
	{
        kDebug(5008) << "Doing startup" << endl;
        pageStartup->defaultConfig();
	}
	else if(current == pageItemColors)
	{
        kDebug(5008) << "Doing colors" << endl;
        pageColors->defaultConfig();
	}
	else if(current == pageItemIRCColors)
	{
        kDebug(5008) << "Doing IRC colors" << endl;
        pageIRCColors->defaultConfig();
	}
	else if(current == pageItemRMBMenu)
	{
		kDebug(5008) << "Doing RMBMenu" << endl;
        pageRMBMenu->defaultConfig();
	}
	else if(current==pageItemServChan)
	{
        kDebug(5008) << "Doing ServerChan" << endl;
        pageServChan->defaultConfig();
	}
	else if(current == pageItemAutoConnect)
	{
        kDebug(5008) << "Doing AutoConnect" << endl;
        pageAutoConnect->defaultConfig();
	}
	else if(current==pageItemFont)
	{
        kDebug(5008) << "Doing font" << endl;
        pageFont->defaultConfig();
	}
	else if(current == pageItemShortcuts)
	{
        kDebug(5008) << "Doing shortcuts" << endl;
        pageShortcuts->defaultConfig();
    }
}

void KSPrefs::modified()
{
    if ( sender() == pageGeneral )
        dirty |= static_cast<int>( KSOptions::General );
    if ( sender() == pageStartup ) {
	dirty |= static_cast<int>( KSOptions::Startup );
	dirty |= static_cast<int>( KSOptions::Servers );
    }
    if ( sender() == pageColors )
        dirty |= static_cast<int>( KSOptions::Colors );
    if ( sender() == pageIRCColors )
        dirty |= static_cast<int>( KSOptions::Colors );
    if ( sender() == pageFont )
        dirty |= static_cast<int>( KSOptions::Colors );
    if ( sender() == pageRMBMenu )
        dirty |= static_cast<int>( KSOptions::RMBMenu );
    if ( sender() == pageServChan ) {
        dirty |= 1;
        pSCDirty = 1;
    }
    if ( sender() == pageAutoConnect ) {
        dirty |= 1;
        pACDirty = 1;
    }
    if ( sender() == pageLooknFeel ) {
        dirty |= 1;
        pLFDirty = 1;
    }
    if ( sender() == pageShortcuts ) {
        dirty |= 1;
        pShortDirty = 1;
    }

    enableButtonApply( dirty );
}
