
#include <kapplication.h>
#include <kconfig.h>

#include <kuser.h>

#include <kdebug.h>

#include "ksopts.h"
//Added by qt3to4:
#include <QPixmap>
#include <kglobal.h>


KSOptions *KSOptions::s_options = 0;
bool KSOChannel::applyGlobally = false;

KSOGeneral::KSOGeneral()
{
    displayMode = MDI;
    autoCreateWin = true;
    autoCreateWinForNotice = false;
    nickCompletion = true;
    displayTopic = true;
    oneLineEntry = true;
    runDocked = false;
//    timeStamp = true;
//    beepNotify = true;
    colorPicker = true;
    autoRejoin = false;
    windowLength = 200;
//    logging = false;
//    beepOnMsg = false;
    publicAway = true;
    //    topicShow = true;
    useColourNickList = false;
    dockPopups = true;
    autoSaveHistory = true;

}

QPixmap KSOGeneral::backgroundPixmap()
{
    if ( m_backgroundPixmap.isNull() && !backgroundFile.isEmpty() )
	m_backgroundPixmap = QPixmap( backgroundFile );
    return m_backgroundPixmap;
}

KSOStartup::KSOStartup()
{
}

KSOServer::KSOServer()
{
    KUser user;
    nick = user.loginName();
    userID = user.loginName();
    realName = user.fullName();
}


KSOColors::KSOColors()
{
    textColor = Qt::black;
    linkColor = Qt::blue;
    infoColor = Qt::blue;
    channelColor = Qt::green;
    errorColor = Qt::red;

    ksircColors = true;
    mircColors = false;
    nickColourization = false;
    ownNickBold = false;
    ownNickUl = true;
    ownNickRev = false;

    msg1Regex = false;
    msg2Regex = false;

    backgroundColor = kapp->palette().color( QPalette::Active, QPalette::Mid );
    selBackgroundColor = kapp->palette().color( QPalette::Active, QPalette::Highlight );
    selForegroundColor = kapp->palette().color( QPalette::Active, QPalette::HighlightedText );

    ircColors[0] = Qt::white;
    ircColors[1] = Qt::black;
    ircColors[2] = Qt::darkBlue;
    ircColors[3] = Qt::darkGreen;
    ircColors[4] = Qt::red;
    ircColors[5] = QColor ( 165, 42, 42 );
    ircColors[6] = Qt::darkMagenta;
    ircColors[7] = QColor( 255, 0x7b, 00 );
    ircColors[8] = Qt::yellow;
    ircColors[9] = Qt::green;
    ircColors[10] = Qt::darkCyan;
    ircColors[11] = Qt::cyan;
    ircColors[12] = Qt::blue;
    ircColors[13] = QColor( 255, 0, 255 );
    ircColors[14] = Qt::gray;
    ircColors[15] = Qt::lightGray;

    uint i;
    for(i = 0; i < numColors; i++){
        nickHighlight[i] = true;
    }

    colourTheme = "Custom";
}

KSORMBMenu::KSORMBMenu()
{
}

KSOServChan::KSOServChan()
{
}

KSOChannel::KSOChannel()
{
    timeStamp = false;
    beepNotify = false;
    beepOnMsg = false;
    logging = false;
    topicShow = true;
    filterJoinPart = false;
    applyGlobally = false;
    encoding = "Default";
}

void KSOptions::load( int sections )
{
    KConfig *conf = KGlobal::config();
    conf->setGroup( "ServerController" );
    geometry = conf->readEntry( "Size",QRect() );

    // Using the fields itself as default values as they are already
    // set to the defaults.
    if ( sections & General )
    {
        conf->setGroup( "General" );
        int mode = conf->readEntry( "DisplayMode", static_cast<int>(displayMode) );
        if (mode == 0)
            displayMode = SDI;
        else
            displayMode = MDI;

        autoCreateWin = conf->readEntry( "AutoCreateWin", autoCreateWin );
        autoCreateWinForNotice = conf->readEntry( "AutoCreateWinForNotice", autoCreateWinForNotice );
        nickCompletion = conf->readEntry( "NickCompletion", nickCompletion );
        displayTopic = conf->readEntry( "DisplayTopic", displayTopic );
	oneLineEntry = conf->readEntry( "OneLineEntry", oneLineEntry );
	runDocked = conf->readEntry( "runDocked", runDocked );
//        timeStamp = conf->readEntry( "TimeStamp", timeStamp );
//        beepNotify = conf->readEntry( "BeepNotify", beepNotify );
        colorPicker = conf->readEntry( "ColourPicker", colorPicker );
        autoRejoin = conf->readEntry( "AutoRejoin", autoRejoin );
        windowLength = conf->readEntry( "WindowLength", windowLength );
        backgroundFile = conf->readPathEntry( "BackgroundFile", backgroundFile );
//        logging = conf->readEntry( "Logging", logging );
//      beepOnMsg = conf->readEntry( "BeepOnMessage", false );
        publicAway = conf->readEntry( "PublicAway", publicAway );
        //      topicShow = conf->readEntry("TopicShow", topicShow);
	useColourNickList = conf->readEntry("ColourNickList", useColourNickList);
	dockPopups = conf->readEntry("dockPopups", dockPopups);
        autoSaveHistory = conf->readEntry("autoSaveHistory", autoSaveHistory);
    }

    if ( sections & Startup )
    {
	conf->setGroup( "StartUp" );
    }


    if ( sections & Colors )
    {
        if ( !conf->hasGroup( "ColorScheme" ) && conf->hasGroup( "Colours" ) )
        {
            conf->setGroup( "Colours" );
            textColor = conf->readEntry( "text", textColor );
            infoColor = conf->readEntry( "info", infoColor );
            channelColor = conf->readEntry( "chan", channelColor );
            errorColor = conf->readEntry( "error", errorColor );
            int c = conf->readEntry( "uscolour", -1 );
            if ( c >= 0 && c < 16 )
                ownNickColor = ircColors[c];
            c = conf->readEntry( "nickfcolour", -1 );
            if ( c >= 0 && c < 16 )
                nickForeground = ircColors[c];
            c = conf->readEntry( "nickbcolour", -1 );
            if ( c >= 0 && c < 16 )
                nickBackground = ircColors[c];
            backgroundColor = conf->readEntry( "Background", backgroundColor );
        }
        else
        {
            conf->setGroup( "ColorScheme" );
            textColor = conf->readEntry( "Text", textColor );
            linkColor = conf->readEntry( "Link", linkColor );
            infoColor = conf->readEntry( "Info", infoColor );
            channelColor = conf->readEntry( "Channel", channelColor );
            errorColor = conf->readEntry( "Error", errorColor );
	    ownNickColor = conf->readEntry( "OwnNick", ownNickColor );
	    ownNickBold = conf->readEntry( "OwnNickBold", ownNickBold );
	    ownNickUl = conf->readEntry( "OwnNickUnderline", ownNickUl );
            ownNickRev = conf->readEntry( "OwnNickReverse", ownNickRev );
	    msgContainNick = conf->readEntry( "msgContainNick", ownNickColor ); // default to the ownNick colour
	    msg1Contain = conf->readEntry( "msg1Contain", msg1Contain );
	    msg1String = conf->readEntry( "msg1String", msg1String );
            msg1Regex = conf->readEntry( "msg1Regex", msg1Regex );
	    msg2Contain = conf->readEntry( "msg2Contain", msg2Contain );
	    msg2String = conf->readEntry( "msg2String", msg2String );
            msg2Regex = conf->readEntry( "msg2Regex", msg2Regex );
            nickForeground = conf->readEntry( "NickForeground", nickForeground );
            nickBackground = conf->readEntry( "NickBackground", nickBackground );
            backgroundColor = conf->readEntry( "Background", backgroundColor );
            selBackgroundColor = conf->readEntry( "SelBackground", selBackgroundColor );
	    selForegroundColor = conf->readEntry( "SelForeground", selForegroundColor );
	    ksircColors = conf->readEntry( "ksircColors", ksircColors );
	    mircColors = conf->readEntry( "mircColors", mircColors );
	    nickColourization = conf->readEntry( "nickColourization", mircColors );
            colourTheme = conf->readEntry("ColourTheme", colourTheme );
	    for ( int i = 0; i < 16; ++i ) {
		ircColors[i] =
		    conf->readEntry(QString::fromLatin1( "IRC-%1" ).arg( i ),
					 ircColors[i] );
                nickHighlight[i] =
		    conf->readEntry(QString::fromLatin1( "nickHighlight-%1" ).arg( i ),
					 nickHighlight[i] );
	    }
        }
	conf->setGroup( "GlobalOptions" );
        QFont df = kapp->font(0);
        defaultFont = conf->readEntry( "MainFont", df);
        QApplication::setFont( defaultFont, true, "KSirc::TextView" );
    }
    if ( sections & Channels )
    {
        conf->setGroup( "Channel" );

        /*
         * do global first
         */
        if(channel.contains("global") == FALSE){
            KSOChannel global;
            ChannelOpMap coMap;
            channel.insert(QString("global"), coMap);
            channel["global"].insert("global", global);
            channel["global"]["global"].server = "global";
            channel["global"]["global"].channel = "global";
            channel["global"]["global"].timeStamp = conf->readEntry("globalglobalTimeStamp", false);
            channel["global"]["global"].beepNotify = conf->readEntry("globalglobalBeepNotify", false);
            channel["global"]["global"].beepOnMsg = conf->readEntry("globalglobalBeepOnMsg", false);
            channel["global"]["global"].logging = conf->readEntry("globalglobalLogging", false);
	    channel["global"]["global"].topicShow = conf->readEntry("globalglobalTopicShow", true);
	    channel["global"]["global"].filterJoinPart = conf->readEntry("globalglobalFilterJoinPart", false);
	    channel["global"]["global"].applyGlobally = conf->readEntry("globalglobalApplyGlobally", false);
	    channel["global"]["global"].encoding = conf->readEntry("globalglobalEncoding", "Default");
        }

	QDateTime now(QDateTime::currentDateTime());
        QStringList servers = conf->readEntry("Servers",QStringList());
        QStringList::ConstIterator ser = servers.begin();
        for( ; ser != servers.end(); ser++){
            ChannelOpMap coMap;
            channel.insert(*ser, coMap);
            QStringList channels = conf->readEntry(*ser, QStringList());
            QStringList::ConstIterator chan = channels.begin();
	    for( ; chan != channels.end(); chan++){
		QDateTime lastUsed = conf->readDateTimeEntry(*ser + *chan + "lastUsed", &now);
		if((lastUsed.daysTo(now) < 30 )||
		   (*chan == "global" && *ser == "global"))
		{
		    KSOChannel ksoc;
		    ksoc = channel["global"]["global"];
		    ksoc.server = *ser;
		    ksoc.channel = *chan;
		    ksoc.timeStamp = conf->readEntry(*ser + *chan + "TimeStamp", ksoc.timeStamp);
		    ksoc.beepNotify = conf->readEntry(*ser + *chan + "BeepNotify", ksoc.beepNotify);
		    ksoc.beepOnMsg = conf->readEntry(*ser + *chan + "BeepOnMsg", ksoc.beepOnMsg);
		    ksoc.logging = conf->readEntry(*ser + *chan + "Logging", ksoc.logging);
		    ksoc.topicShow = conf->readEntry(*ser + *chan + "TopicShow", ksoc.topicShow);
		    ksoc.encoding = conf->readEntry(*ser + *chan + "Encoding", ksoc.encoding);
		    ksoc.filterJoinPart = conf->readEntry(*ser + *chan + "FilterJoinPart", ksoc.filterJoinPart);
                    ksoc.lastUsed = lastUsed;
		    channel[*ser].insert(*chan, ksoc);
		}
		else { /* expire junk */
		    conf->deleteEntry(*ser + *chan + "TimeStamp");
		    conf->deleteEntry(*ser + *chan + "BeepNotify");
		    conf->deleteEntry(*ser + *chan + "BeepOnMsg");
		    conf->deleteEntry(*ser + *chan + "Logging");
		    conf->deleteEntry(*ser + *chan + "TopicShow");
		    conf->deleteEntry(*ser + *chan + "Encoding");
		    conf->deleteEntry(*ser + *chan + "FilterJoinPart");
                    conf->deleteEntry(*ser + *chan + "lastUsed");
		}
            }
        }
    }
    if ( sections & Servers )
    {
	conf->setGroup( "Server" );

	/*
	 * do global first
	 */
	if(server.contains("global") == FALSE){
	    KSOServer global;
	    server.insert(QString("global"), global);
	    server["global"].server = "global";
	    server["global"].globalCopy = false;

	    conf->setGroup( "StartUp" );
	    server["global"].nick = conf->readEntry("Nick",QString());
	    server["global"].altNick = conf->readEntry("altNick",QString());
	    server["global"].realName = conf->readEntry("RealName",QString());
	    server["global"].userID = conf->readEntry("userID",QString());
	    server["global"].notifyList = conf->readEntry("NotifyList",QStringList());

	    conf->setGroup( "Server" );
	}

	kDebug(5008) << "Load: in Server" << endl;

	QStringList servers = conf->readEntry("Servers",QStringList());

	QStringList::ConstIterator ser = servers.begin();
	for( ; ser != servers.end(); ser++){
	    KSOServer ksos;
	    ksos = server["global"];
	    ksos.server = *ser;
	    ksos.globalCopy = false;
	    ksos.nick = conf->readEntry(*ser + "nick",QString());
	    ksos.altNick = conf->readEntry(*ser + "altNick",QString());
	    ksos.realName = conf->readEntry(*ser + "realName",QString());
	    ksos.userID = conf->readEntry(*ser + "userID",QString());
	    ksos.notifyList = conf->readEntry(*ser + "notifyList",QStringList());
	    kDebug(5008) << *ser << ": nick: " << ksos.nick << " altNick: " << ksos.altNick << endl;
	    server.insert(*ser, ksos);
	}
    }
    kDebug(5008) << "Done full load" << endl;
}

void KSOptions::save( int sections )
{
    KConfig *conf = KGlobal::config();

    if ( sections & General )
    {
        conf->setGroup( "General" );
        conf->writeEntry( "DisplayMode", static_cast<int>(displayMode) );
        conf->writeEntry( "AutoCreateWin", autoCreateWin );
        conf->writeEntry( "AutoCreateWinForNotice", autoCreateWinForNotice );
        conf->writeEntry( "NickCompletion", nickCompletion );
        conf->writeEntry( "DisplayTopic", displayTopic );
	conf->writeEntry( "OneLineEntry", oneLineEntry );
	conf->writeEntry( "runDocked", runDocked );
//      conf->writeEntry( "TimeStamp", timeStamp );
//      conf->writeEntry( "BeepNotify", beepNotify );
        conf->writeEntry( "ColourPicker", colorPicker );
        conf->writeEntry( "AutoRejoin", autoRejoin );
        conf->writeEntry( "WindowLength", windowLength );
        conf->writePathEntry( "BackgroundFile", backgroundFile );
//      conf->writeEntry( "Logging", logging );
//      conf->writeEntry( "BeepOnMessage", beepOnMsg );
        conf->writeEntry( "PublicAway", publicAway );
	conf->writeEntry( "ColourNickList", useColourNickList );
	conf->writeEntry( "dockPopups", dockPopups );
        conf->writeEntry( "autoSaveHistory", autoSaveHistory );
    }

    if ( sections & Startup )
    {
	conf->setGroup( "StartUp" );
        /*
	conf->writeEntry( "Nick", nick );
	conf->writeEntry( "AlternativeNick", altNick );
	conf->writeEntry( "RealName", realName );
	conf->writeEntry( "userID", userID );
	conf->writeEntry( "NotifyList", notifyList );
        */
    }

    if ( sections & Colors )
    {
        conf->setGroup( "ColorScheme" );
        conf->writeEntry( "Text", textColor );
        conf->writeEntry( "Link", linkColor );
        conf->writeEntry( "Info", infoColor );
        conf->writeEntry( "Channel", channelColor );
        conf->writeEntry( "Error", errorColor );
	conf->writeEntry( "OwnNick", ownNickColor );
	conf->writeEntry( "OwnNickBold", ownNickBold );
	conf->writeEntry( "OwnNickUnderline", ownNickUl );
	conf->writeEntry( "OwnNickReverse", ownNickRev );
	conf->writeEntry( "msgContainNick", msgContainNick );
	conf->writeEntry( "msg1Contain", msg1Contain );
	conf->writeEntry( "msg1String", msg1String );
	conf->writeEntry( "msg1Regex", msg1Regex );
	conf->writeEntry( "msg2Contain", msg2Contain );
	conf->writeEntry( "msg2String", msg2String );
	conf->writeEntry( "msg2Regex", msg2Regex );
        conf->writeEntry( "NickForeground", nickForeground );
        conf->writeEntry( "NickBackground", nickBackground );
        conf->writeEntry( "Background", backgroundColor );
        conf->writeEntry( "SelBackground", selBackgroundColor );
	conf->writeEntry( "SelForeground", selForegroundColor );
	conf->writeEntry( "ksircColors", (bool) ksircColors );
	conf->writeEntry( "mircColors", mircColors );
	conf->writeEntry( "nickColourization", nickColourization );
        conf->writeEntry( "ColourTheme", colourTheme );
	for ( int i = 0; i < 16; ++i ) {
	    conf->writeEntry( QString::fromLatin1( "IRC-%1" ).arg( i ), ircColors[i] );
	    conf->writeEntry( QString::fromLatin1( "nickHighlight-%1" ).arg( i ), nickHighlight[i] );
	}
        /* Store the font setting */
        conf->setGroup( "GlobalOptions" );
        conf->writeEntry("MainFont", defaultFont);
        /* end of storing the font settings */
    }
    if ( sections & Channels )
    {
	QStringList servers;

	conf->deleteGroup( "Channel" );

        conf->setGroup( "Channel" );
        ServerChannelOpMap::Iterator ser;
        for( ser = channel.begin(); ser != channel.end(); ++ser ){
            QStringList channels;

            servers << ser.key();

            ChannelOpMap::Iterator chan;
            for( chan = ser.data().begin(); chan != ser.data().end(); ++chan ){
		channels << chan.key();
                conf->writeEntry(ser.key() + chan.key() + "TimeStamp", chan.data().timeStamp);
                conf->writeEntry(ser.key() + chan.key() + "BeepNotify", chan.data().beepNotify);
                conf->writeEntry(ser.key() + chan.key() + "BeepOnMsg", chan.data().beepOnMsg);
                conf->writeEntry(ser.key() + chan.key() + "Logging", chan.data().logging);
		conf->writeEntry(ser.key() + chan.key() + "TopicShow", chan.data().topicShow);
		conf->writeEntry(ser.key() + chan.key() + "Encoding", chan.data().encoding);
		conf->writeEntry(ser.key() + chan.key() + "FilterJoinPart", chan.data().filterJoinPart);
		conf->writeEntry(ser.key() + chan.key() + "lastUsed", chan.data().lastUsed);
            }
            conf->writeEntry(ser.key(), channels);
        }
        conf->writeEntry("Servers", servers);
        conf->writeEntry("globalglobalApplyGlobally", channel["global"]["global"].applyGlobally);
    }
    if ( sections & Servers )
    {
	QStringList servers;

	conf->deleteGroup( "Servers" );

        conf->setGroup( "Server" );
        ServerOpMap::Iterator ser;
        for( ser = server.begin(); ser != server.end(); ++ser ){
	    QStringList channels;

	    if(ser.data().globalCopy == true)
                continue;

            servers << ser.key();

	    conf->writeEntry(ser.key() + "nick", ser.data().nick);
	    conf->writeEntry(ser.key() + "altNick", ser.data().altNick);
	    conf->writeEntry(ser.key() + "realName", ser.data().realName);
	    conf->writeEntry(ser.key() + "userID", ser.data().userID);
	    conf->writeEntry(ser.key() + "notifyList", ser.data().notifyList);
	}
	conf->writeEntry("Servers", servers);
    }

    conf->sync();
}

void KSOptions::channelSetup(QString serv, QString chan)
{
    if(channel.contains(serv) == FALSE){
        ChannelOpMap coMap;
        channel.insert(serv, coMap);
    }
    if(channel[serv].contains(chan) == FALSE){
        KSOChannel ksoc;
        ksoc = channel["global"]["global"];
        channel[serv].insert(chan, ksoc);
        channel[serv][chan].server = serv;
	channel[serv][chan].channel = chan;
	channel[serv][chan].lastUsed = QDateTime::currentDateTime();
    }
    else {
	channel[serv][chan].lastUsed = QDateTime::currentDateTime();
    }
}

void KSOptions::applyChannelGlobal(void)
{
    ServerChannelOpMap::Iterator ser;
    for( ser = channel.begin(); ser != channel.end(); ++ser ){
        ChannelOpMap::Iterator chan;
        for( chan = ser.data().begin(); chan != ser.data().end(); ++chan ){
            if((chan.key() == "global") && (ser.key() == "global"))
                continue;

            chan.data() = channel["global"]["global"];
        }
    }

}

void KSOptions::serverSetup(QString serv)
{
    if(server.contains(serv) == FALSE){
        KSOServer ksos;
        ksos = server["global"];
        server.insert(serv, ksos);
	server[serv].server = serv;
        server[serv].globalCopy = true;
	server[serv].lastUsed = QDateTime::currentDateTime();
    }
    else {
	server[serv].lastUsed = QDateTime::currentDateTime();
    }
}


/* vim: et sw=4 ts=4
 */
