#ifndef ksview_h
#define ksview_h

/* This file is part of ksirc
   Copyright (c) 2001 Malte Starostik <malte@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

// $Id: ksview.h 467420 2005-10-05 09:03:00Z mueller $
//
#include <QTextBrowser>
//Added by qt3to4:
#include <QDragEnterEvent>
#include <QMouseEvent>
#include <QDragMoveEvent>
#include <QDropEvent>

class KSircProcess;

class KSircView : public QTextBrowser
{
	Q_OBJECT
public:
	KSircView(KSircProcess *proc, QWidget *parent, const char *name);
	virtual ~KSircView();

	void clear();
	// returns the plain-text string added (for logging)
	QString addLine(const QString &pixmap, const QColor &color, const QString &text);
        void addRichText(const QString &_text);
        void enableTimeStamps(bool enable);

	void setAcceptFiles(bool _acceptFiles) { m_acceptFiles = _acceptFiles; }

	void scrollToBottom();
signals:
        void urlsDropped(const QStringList&);
        void textDropped(const QString&);

protected:
        virtual void dragEnterEvent(QDragEnterEvent *);
        virtual void dragMoveEvent(QDragMoveEvent *);
        virtual void dropEvent(QDropEvent *);

    virtual QVariant loadResource(int type, const QUrl &name);
	virtual void resizeEvent(QResizeEvent *e);
	virtual void contextMenuEvent(QContextMenuEvent *e);

private:
	virtual bool linksEnabled() const { return true; }
    void openBrowser( const QString &url );
    void copyLinkToClipboard( const QString &url );
	static QString makeTimeStamp();

	void saveURL( const QString &url );

private slots:
	void activateAnchor(const QUrl &url);

	QColor ircColor(int);

private:
	int m_lines;
	QStringList m_timeStamps;
        bool m_acceptFiles;
	bool m_timestamps;
        KSircProcess *m_proc;
};

// vim: ts=4 sw=4 noet

#endif
