#include "objFinder.h"

#include <qapplication.h>
#include <qobject.h>
#include <qwidget.h>

#include <stdlib.h>
#include <time.h>

#include <kdebug.h>

Q3Dict<QObject> *objFinder::objList = new Q3Dict<QObject>;

/*
 * So we can connect to the slots, etc
 */
objFinder *objFind = new objFinder();

objFinder::objFinder()
  : QObject()
{
}

objFinder::~objFinder()
{
}

void objFinder::insert(QObject *obj, const char *key){
  QString name;

  if(obj == 0x0){
    qWarning("objFinder: Passed Null Object");
    return;
  }
  
  if(key != 0){
    name = key;
  }
  else {
    name = obj->objectName();
    if(name.isEmpty()){
      name = randString();
    }
  }
  objList->insert(name, obj);
  connect(obj, SIGNAL(destroyed()),
          objFind, SLOT(objDest()));
  
  emit objFind->inserted(obj);
}

QObject *objFinder::find(const char *name, const char *inherits){
  Q3DictIterator<QObject> it(*objList);
  uint len = strlen(name);
  while(it.current()){
    if(len == strlen(it.current()->name()) &&
       strcmp(it.current()->name(), name) == 0)
      return it.current();
    QObjectList qobl = it.current()->queryList(inherits, name, FALSE);
    if (!qobl.isEmpty())
	return qobl.first();
    ++it;
  }
  QWidgetList all = QApplication::allWidgets();
  foreach(QWidget* widget, all) {
    if(len == widget->objectName().length() &&
       widget->objectName() == QLatin1String(name)){
      if(inherits != 0x0 && widget->inherits(inherits) == FALSE) {
        continue;
      }
      return widget;
    }
  }
  
  return 0x0;
}

void objFinder::dumpTree(){
  Q3DictIterator<QObject> it(*objList);
  while(it.current()){
    it.current()->dumpObjectTree();
    ++it;
  }
  QWidgetList all = QApplication::allWidgets();
  foreach(QWidget* widget, all) {
    kDebug(5008) << widget->className() << "::" << widget->name("unnamed") << endl;
  }

}

QStringList objFinder::allObjects(){
  QStringList allNames;
  Q3DictIterator<QObject> it(*objList);
  while(it.current()){
    QObjectList qobl = it.current()->queryList(); // Matches everything
    foreach(QObject* obj, qobl){
      QString name;
      name = obj->className();
      name += "::";
      name += obj->name("unnamed");
      allNames.append(name);
    }
    ++it;
  }
  QWidgetList all = QApplication::allWidgets();
  foreach(QWidget* widget, all){
    QString name;
    name = widget->className();
    name += "::";
    name += widget->name("unnamed");
    allNames.append(name);
  }
  return allNames;
}

QString objFinder::randString(){
  static bool runSrand = 0;
  QString str = "";
  if(runSrand == 0){
    srand(time(NULL));
  }
  for(int i = 0; i <= 8; i++){
    str.insert(0, (char) (1+(int) (94.0*rand()/(RAND_MAX+1.0))) + 0x20);
  }
  return str;
}

void objFinder::objDest(){
  if(sender() == 0x0){
    return;
  }
  Q3DictIterator<QObject> it(*objList);
  while(it.current()){
    if(it.current() == sender()){
      objList->remove(it.currentKey());
    }
    ++it;
  }
}
#include "objFinder.moc"
